!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine Green_terms(R,Z,R0,Z0,Green_val,Green_R,Green_Z)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! returns the value of the Green function and its gradients with respect to the observation point R0,Z0

	use mod_parameters, only : dkind, pi

	implicit none

	real(kind=dkind), intent(in) :: R,Z,R0,Z0
	real(kind=dkind) :: Green_val,Green_R,Green_Z
	real(kind=dkind) :: k_ell, e_ell
	real(kind=dkind) :: elle, ellf
	real(kind=dkind) :: k, k2, kov2, Rdiff2, Zdiff2, term2, den

	Rdiff2 = (R-R0)**2
	Zdiff2 = (Z-Z0)**2

	k2 = 4.d0*R*R0/((R+R0)**2+Zdiff2)
	k = sqrt(k2)
	kov2 = k/2.d0

	k_ell = ellf(0.5d0*pi,k)
	e_ell = elle(0.5d0*pi,k)

	Green_val = - 1.d0/(2.d0*pi) * sqrt(R*R0) * 1.d0/k *  &
						((2.d0-k2)*k_ell - 2.d0*e_ell)

	term2 = (Rdiff2 +Zdiff2)*k_ell
	den = 2.d0*pi*sqrt(R*R0)*(Rdiff2 + Zdiff2)

	Green_R = -(R0*kov2 * ((R**2 - R0**2 - Zdiff2)*e_ell +  &
!							(Rdiff2 +Zdiff2)*k_ell))/(2.d0*pi*sqrt(R*R0)*(Rdiff2 + Zdiff2))
							term2)) / den

	Green_Z = -(kov2*(Z - Z0)*((R**2 + R0**2 +Zdiff2)*e_ell -  &
!							(Rdiff2 + Zdiff2)*k_ell))/(2.d0*pi*sqrt(R*R0)*(Rdiff2 + Zdiff2))
							term2) )/ den

	continue


end subroutine Green_terms

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine Green_single_term(R,Z,R0,Z0,Green_val)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! returns the value of the Green function and its gradients with respect to the observation point R0,Z0

	use mod_parameters, only : dkind, pi

	implicit none

	real(kind=dkind), intent(in) :: R,Z,R0,Z0
	real(kind=dkind) :: Green_val
	real(kind=dkind) :: k_ell, e_ell
	real(kind=dkind) :: elle, ellf
	real(kind=dkind) :: k, k2, Rdiff2, Zdiff2, term2, den

	Rdiff2 = (R-R0)**2
	Zdiff2 = (Z-Z0)**2

	k2 = 4.d0*R*R0/((R+R0)**2+Zdiff2)
	k = sqrt(k2)

	k_ell = ellf(0.5d0*pi,k)
	e_ell = elle(0.5d0*pi,k)

	Green_val = - 1.d0/(2.d0*pi) * sqrt(R*R0) * 1.d0/k *  &
						((2.d0-k2)*k_ell - 2.d0*e_ell)

	continue


end subroutine Green_single_term

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
function Green(R,Z,R0,Z0) result(answer)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! returns the value of the Green function only with respect to the observation point R0,Z0

	use mod_parameters, only : dkind, pi

	implicit none

	real(kind=dkind) :: R, Z, R0, Z0
	real(kind=dkind) :: answer
	real(kind=dkind) :: k_ell, e_ell
	real(kind=dkind) :: k_ell1, e_ell1, k_ell2, e_ell2
	real(kind=dkind) :: elle, ellf
	real(kind=dkind) :: k, k2

	k2 = 4.d0*R*R0/((R+R0)**2+(Z-Z0)**2)
	k = sqrt(k2)

	k_ell1 = ellf(0.5d0*pi,k)
	e_ell1 = elle(0.5d0*pi,k)
	k_ell2 = ellf(0.5d0*pi,k2)
	e_ell2 = elle(0.5d0*pi,k2)

	k_ell = k_ell1
	e_ell = e_ell1

!	k_ell = k_ell2
!	e_ell = e_ell2

	answer = - 1.d0/(2.d0*pi) * sqrt(R*R0) * 1.d0/k *  &
						((2.d0-k2)*k_ell - 2.d0*e_ell)

!!$	k2 = 4.d0*rmajor*bigr/((rmajor+bigr)**2+z**2)
!!$	k = 2.d0*sqrt(rmajor*bigr/((rmajor+bigr)**2+z**2))

!!$	k_ell = ellf(0.5d0*pi,k)
!!$	e_ell = elle(0.5d0*pi,k)

	continue

end function Green

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
function rf(x,y,z)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! from numerical recipes

	use mod_parameters, only : dkind

	implicit none

	real(kind=dkind) ::  rf,x,y,z,errtol,tiny,big,third,c1,c2,c3,c4
	parameter (errtol=.0025d0,tiny=1.5d-38,big=3.d37,third=1.d0/3.d0,  &
				c1=1.d0/24.d0,c2=.1d0,c3=3.d0/44.d0,c4=1.d0/14.d0)
	real (kind=dkind) :: alamb,ave,delx,dely,delz,e2,e3,sqrtx,sqrty,sqrtz,xt,yt,zt

	if(min(x,y,z).lt.0.d0.or.min(x+y,x+z,y+z).lt.tiny.or.max(x,y,  &
				z).gt.big)pause 'invalid arguments in rf'

	xt=x
	yt=y
	zt=z

	1     continue

		sqrtx=sqrt(xt)
		sqrty=sqrt(yt)
		sqrtz=sqrt(zt)
		alamb=sqrtx*(sqrty+sqrtz)+sqrty*sqrtz
		xt=.25d0*(xt+alamb)
		yt=.25d0*(yt+alamb)
		zt=.25d0*(zt+alamb)
		ave=third*(xt+yt+zt)
		delx=(ave-xt)/ave
		dely=(ave-yt)/ave
		delz=(ave-zt)/ave

	if(max(abs(delx),abs(dely),abs(delz)).gt.errtol)goto 1
	e2=delx*dely-delz**2
	e3=delx*dely*delz
	rf=(1.d0+(c1*e2-c2-c3*e3)*e2+c4*e3)/sqrt(ave)

	return

end function rf

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
function rd(x,y,z)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! from numerical recipes

	use mod_parameters, only : dkind

	implicit none

	real(kind=dkind) :: rd,x,y,z,errtol,tiny,big,c1,c2,c3,c4,c5,c6
	parameter (errtol=.0015d0,tiny=1.d-25,big=4.5d21,c1=3.d0/14.d0,  &
			c2=1.d0/6.d0,c3=9.d0/22.d0,c4=3.d0/26.d0,c5=.25d0*c3,c6=1.5d0*c4)
	real(kind=dkind) :: alamb,ave,delx,dely,delz,ea,eb,ec,ed,ee,fac,  &
					sqrtx,sqrty,sqrtz,sum,xt,yt,zt

	if(min(x,y).lt.0.d0.or.min(x+y,z).lt.tiny.or.max(x,y,z).gt.big)  &
				pause 'invalid arguments in rd'

	xt=x
	yt=y
	zt=z
	sum=0.d0
	fac=1.d0

	1     continue

		sqrtx=sqrt(xt)
		sqrty=sqrt(yt)
		sqrtz=sqrt(zt)
		alamb=sqrtx*(sqrty+sqrtz)+sqrty*sqrtz
		sum=sum+fac/(sqrtz*(zt+alamb))
		fac=.25d0*fac
		xt=.25d0*(xt+alamb)
		yt=.25d0*(yt+alamb)
		zt=.25d0*(zt+alamb)
		ave=.2d0*(xt+yt+3.d0*zt)
		delx=(ave-xt)/ave
		dely=(ave-yt)/ave
		delz=(ave-zt)/ave

	if(max(abs(delx),abs(dely),abs(delz)).gt.errtol)goto 1

	ea=delx*dely
	eb=delz*delz
	ec=ea-eb
	ed=ea-6.d0*eb
	ee=ed+ec+ec
	rd=3.d0*sum+fac*(1.d0+ed*(-c1+c5*ed-c6*delz*ee)+delz*(c2*ee+delz*(-c3*  &
			ec+delz*c4*ea)))/(ave*sqrt(ave))

	return

end function rd


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
function elle(phi,ak)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! from numerical recipes
!!    uses rd,rf

	use mod_parameters, only : dkind

	implicit none

	real(kind=dkind) :: elle,ak,phi
	real(kind=dkind) :: cc,q,s,rd,rf

	s=sin(phi)
	cc=cos(phi)**2
	q=(1.d0-s*ak)*(1.+s*ak)
	elle=s*(rf(cc,q,1.d0)-((s*ak)**2)*rd(cc,q,1.d0)/3.d0)

	return

end function elle

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
function ellf(phi,ak)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! from numerical recipes
!    uses rf

	use mod_parameters, only : dkind

	implicit none

	real(kind=dkind) :: ellf,ak,phi
	real(kind=dkind) :: s,rf

	s=sin(phi)
	ellf=s*rf(cos(phi)**2,(1.d0-s*ak)*(1.d0+s*ak),1.d0)

	return

end function ellf
