module mod_boundary

	use mod_parameters, only : dkind

	implicit none

	real(kind=dkind), dimension(:,:), allocatable :: output_boundary_coord
	! box coordinates

	real(kind=dkind), dimension(:), allocatable :: psi_boundary
	! the goal of the calculation (psi on a box)

	integer :: n_boundary
	integer :: n_side	! points per side

	real(kind=dkind) :: dx_edge, dz_edge

	real(kind=dkind) :: psi_offset

	contains

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine get_boundary_psi
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! returns psi on output boundary

	use mod_parameters, only : mu_mag, n_enforce, pi, n_fit
	use mod_transversal_commons, only : fixed_coords
	use mod_plasma, only : radius_theta

	real(kind=dkind) :: R_loc, Z_loc, psi_loc
!	real(kind=dkind) :: temp_psi, temp_Rder, temp_Zder
!	real(kind=dkind) :: psi_boundary_fun
	real(kind=dkind) :: rloc, thetaloc
	integer :: i

!	call initialize_boundary
	call set_ouput_boundary

	allocate(psi_boundary(max(n_boundary,size(fixed_coords,1))))

!!$	R_loc = fixed_coords(1,1)
!!$	Z_loc = fixed_coords(1,2)
!!$	temp_psi = psi_boundary_fun_2(R_loc,Z_loc)
!!$	temp_Rder = psi_boundary_fun_Rder(R_loc,Z_loc)
!!$	temp_Zder = psi_boundary_fun_Zder(R_loc,Z_loc)
	continue

	! for debugging
	open(69, file='fixed_coords.dat')

	do i = 1, size(fixed_coords,1)

		R_loc = fixed_coords(i,1)
		Z_loc = fixed_coords(i,2)
		psi_loc = psi_boundary_fun(R_loc,Z_loc)
		psi_boundary(i) = psi_loc*mu_mag

		write(69,88) fixed_coords(i,0), R_loc, Z_loc, -psi_boundary(i)

		continue

	enddo

	close(69)

	continue

	continue

	open(69, file='boundary_test.dat')

	do i = 1, n_boundary

		thetaloc = 2.d0*pi/(n_boundary-1.d0)*(i-1.d0)
		call radius_theta(thetaloc,rloc,R_loc,Z_loc)
!		R_loc = output_boundary_coord(i,1)
!		Z_loc = output_boundary_coord(i,2)
		psi_loc = psi_boundary_fun(R_loc,Z_loc)
		psi_boundary(i) = psi_loc*mu_mag

		write(69,88) thetaloc, R_loc, Z_loc, -psi_boundary(i)

		continue

	enddo

	close(69)

	! now actually compute the desired psi
	do i = 1, n_boundary

		R_loc = output_boundary_coord(i,1)
		Z_loc = output_boundary_coord(i,2)
		psi_loc = psi_boundary_fun(R_loc,Z_loc)
		psi_boundary(i) = -psi_loc*mu_mag + psi_offset

		continue

	enddo

88  format(4(e15.9, 3x))

	continue

end subroutine get_boundary_psi


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine set_ouput_boundary
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! defines coordinates of points on output boundary.

	use mod_parameters, only : xbox_L, xbox_R, zbox_T, zbox_B, nbox_R, nbox_Z
	use mod_plasma, only : rcenter, zcenter, x_size, z_size, dx, dz, rmajor, theta_of_RZ

	real(kind=dkind) :: x_L, x_R, z_L, z_R
	real(kind=dkind) :: temp, dummy, rminor

	integer :: i, j

	n_boundary = 2 * (nbox_R + nbox_Z) -4

	allocate(output_boundary_coord(n_boundary,0:2))

!	allocate(dx_a(0:nx+1))
!	allocate(dz_a(0:nz+1))

	x_L = xbox_L
	x_R = xbox_R
	z_L = zbox_B
	z_R = zbox_T

	dx_edge = (x_R-x_L)/(nbox_R-1.d0)
	dz_edge = (z_R-z_L)/(nbox_Z-1.d0)

	! set points in a counterclockwise direction starting from top right,
	! with n_side points on each side. First and last point are identical.

	! top side
	do i = 1, nbox_R

		output_boundary_coord(i,1) = x_R - dx_edge*(i-1)
		output_boundary_coord(i,2) = z_R

	enddo

	! left side
	j = 1
	do i = nbox_R+1, nbox_R+nbox_Z-1

		j = j+1
		output_boundary_coord(i,1) = x_L
		output_boundary_coord(i,2) = z_R - dz_edge*(j-1)

	enddo

	! bottom side
	j = 1
	do i = nbox_R+nbox_Z, 2*nbox_R+nbox_Z-2

		j = j+1
		output_boundary_coord(i,1) = x_L + dx_edge*(j-1)
		output_boundary_coord(i,2) = z_L

	enddo

	! right side
	j = 1
	do i = 2*nbox_R+nbox_Z-1, n_boundary

		j = j+1
		output_boundary_coord(i,1) = x_R
		output_boundary_coord(i,2) = z_L + dz_edge*(j-1)

	enddo

!	i = n_boundary + 1
!	output_boundary_coord(i,1) = output_boundary_coord(1,1)
!	output_boundary_coord(i,2) = output_boundary_coord(1,2)

	do i = 1, n_boundary

		call theta_of_RZ(output_boundary_coord(i,0),output_boundary_coord(i,1),output_boundary_coord(i,2))

	enddo

	continue

	return

end subroutine set_ouput_boundary

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
function psi_boundary_fun(Redge,Zedge) result(answer)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! returns psi on output boundary point with given coordinates.

	use mod_plasma, only : nx, nz, dx, dz, dx_a, dz_a, x_coord, z_coord, sort_grid, j_phi
	use mod_coils, only : n_currents, n_CS, R_coil, Z_coil, I_coil, R_CS, Z_CS, I_CS,  &
						CS_points, coord_CS_point, dA_hat_CS, repeat_index

	real (kind=dkind), intent(in) :: Redge, Zedge
	real(kind=dkind) :: answer

	real(kind=dkind) :: Green_loc, term_loc
	real (kind=dkind) :: dA
	real (kind=dkind) :: bigR, bigZ

	real (kind=dkind) :: Green
	external Green

	integer :: i,j,k

!	real(kind=dkind) :: R_l, R_r, angle
!	real(kind=dkind) :: psi_integral(2)

	! first, get the current and the cross section area

	answer = 0.d0

!	dA = dx*dz

	! first the plasma part
	do j=1, nz
		do i=1, nx

		    if(sort_grid(i,j,0)>=1) then
			! may modify to >1 only

				bigR = x_coord(i) + dx_a(i) -  dx_a(i-1)
				bigZ = z_coord(j) + dz_a(j) -  dz_a(j-1)

				Green_loc = Green(bigR,bigZ,Redge,Zedge)

				dA = 0.25d0*(dx_a(i) + dx_a(i-1)) * (dz_a(j) + dz_a(j-1))

				term_loc = j_phi(i,j) * Green_loc * dA
				answer = answer + term_loc
				! minus sign because points are inside curve

			endif

		enddo
	enddo

	! then the coils

	do i = 1, n_currents
	do j = 1, repeat_index(i)

		bigR = R_coil(i,j)
		bigZ = Z_coil(i,j)

		Green_loc = Green(bigR,bigZ,Redge,Zedge)
		term_loc = I_coil(i) * Green_loc
		answer = answer + term_loc

	enddo
	enddo

	! and finally the central solenoid

	do i = 1, n_CS

		bigR = R_CS(i)
		bigZ = Z_CS(i)

		Green_loc = Green(bigR,bigZ,Redge,Zedge)
		term_loc = I_CS * Green_loc

		answer = answer + term_loc

	enddo

	continue

	return

end function psi_boundary_fun

end module mod_boundary
