module mod_coils

	use mod_parameters, only : dkind, n_coils_max, n_CS_max, n_coils_split_max

	implicit none

	integer :: n_coils, n_currents, n_CS
	! number of coils and of windings in the central solenoid
	! n_currents is the number of independent coils

	integer, parameter :: CS_points_max = 2000
	integer :: CS_points = 200
	! integration points in finite size CS coil

	real(kind=dkind), dimension(1:n_coils_max,1:n_coils_split_max) :: R_coil, Z_coil
	real(kind=dkind), dimension(1:n_coils_max) :: R_coil_temp, Z_coil_temp
	! position of coils; second one is for reading input

	integer, dimension(1:n_coils_max) :: coil_labels
	! numbers the coils; equal numbers mean same current

	integer, dimension(1:n_coils_max) :: repeat_index
	! stores the number of times each coil is repeated (= number of times the same current appears for each current)

	real(kind=dkind), dimension(1:n_coils_max) :: I_coil

	real(kind=dkind), dimension(1:n_CS_max) :: R_CS, Z_CS
	! position of CS coil windings

	real(kind=dkind), dimension(1:n_CS_max) :: dR_CS, dZ_CS
	! thickness of CS

	real(kind=dkind), dimension(1:n_CS_max, 1: CS_points_max, 1:2) :: coord_CS_point

	real(kind=dkind), dimension(1:n_CS_max) :: dA_hat_CS
	! pretty much CS coil area divided by CS_points

	real(kind=dkind) :: I_CS

	!default experimental values
	real(kind=dkind) :: R_CS_NSTX = 0.1144d0
	real(kind=dkind) :: Z_CS_NSTX = -2.1d0
	real(kind=dkind) :: dR_CS_NSTX = 0.0349d0
	real(kind=dkind) :: dZ_CS_NSTX = 4.2d0


	!default experimental values
	real(kind=dkind) :: R_CS_DIII = 0.6875
	real(kind=dkind) :: Z_CS_DIII = -1.74987d0
!	real(kind=dkind) :: Z_CS_DIII = -3.49974d0
	real(kind=dkind) :: dR_CS_DIII = 0.05867
	real(kind=dkind) :: dZ_CS_DIII= 3.49974d0
!	real(kind=dkind) :: dZ_CS_DIII= 6.99948d0

	integer, parameter :: n_coils_D3D = 8
	integer, parameter :: n_CS_D3D = 25

	real(kind=dkind), dimension(1:n_coils_D3D) :: R_coil_D3D_array		= 	(/1.15d0, 1.8d0, 2.7d0, 2.8d0, 2.8d0, 2.7d0, 1.8d0, 1.15d0/)
	real(kind=dkind), dimension(1:n_coils_D3D) :: Z_coil_D3D_array		= 	(/1.95d0, 1.85d0, 1.6d0, 0.6d0, -0.6d0, -1.6d0, -1.85d0, -1.95d0/)
	real(kind=dkind), dimension(1:n_CS_D3D) :: R_CS_D3D_array		= 	(/.65d0, .65d0, .65d0, .65d0, .65d0, .65d0, .65d0, .65d0, .65d0, .65d0,  &
							.65d0, .65d0, .65d0, .65d0, .65d0,.65d0, .65d0, .65d0, .65d0, .65d0, .65d0, .65d0, .65d0, .65d0, .65d0/)
	real(kind=dkind), dimension(1:n_CS_D3D) :: Z_CS_D3D_array		=	(/-1.6798752d0, -1.5398856d0, -1.399896d0, -1.2599064d0, -1.1199168d0,   &
							-0.9799272d0, -0.8399376d0, 0.699948d0, -0.5599584d0, -0.4199688d0, -0.2799792d0, -0.1399896d0, 0d0, 0.1399896d0,  &
							0.2799792d0, 0.4199688d0, 0.5599584d0, 0.699948d0, 0.8399376d0, 0.9799272d0, 1.1199168d0, 1.2599064d0,  &
							1.399896d0, 1.5398856d0, 1.6798752d0/)

end module mod_coils