module mod_input

	use mod_parameters, only : dkind, which_data, n_enforce, nbox_R,nbox_Z, xbox_L, xbox_R, zbox_T, zbox_B,  &
													fixed_coords_option, HYM_length, HYM_current
	use mod_coils, only : n_coils, n_CS, R_coil_temp, Z_coil_temp, coil_labels,  &
										R_CS, Z_CS, dR_CS, dZ_CS
	use mod_plasma, only : x_size, z_size, rcenter, nx, nz, rmajor
	use mod_equation_system, only : solution_method, theta_start, n_fit
	use mod_boundary, only : psi_offset

	implicit none

	namelist/input_basics/	which_data
	namelist/input_geometry/	x_size, z_size, rcenter
	namelist/input_plasma/	nx, nz, rmajor
	namelist/input_coils/	n_coils, n_CS
	namelist/input_box/	nbox_R, nbox_Z, xbox_L, xbox_R, zbox_T, zbox_B
	namelist/input_numerics/ solution_method, theta_start, n_fit, psi_offset, fixed_coords_option
	namelist/input_HYM/ HYM_length, HYM_current

	contains

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine readinput
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	use mod_coils, only : R_CS_NSTX, Z_CS_NSTX, dR_CS_NSTX, dZ_CS_NSTX,  &
										R_CS_DIII, Z_CS_DIII, dR_CS_DIII, dZ_CS_DIII, repeat_index, coil_labels,  &
										R_coil, Z_coil, R_coil_temp, Z_coil_temp, n_currents

	integer :: n_repeated, i_loc, j_loc, label_loc

	integer :: i, j

	open(unit=5,file='FREE_FIX_input.dat',status='old')

	read(5,input_basics)
	read(5,input_geometry)
	read(5,input_plasma)
	read(5,input_coils)
	read(5,input_box)
	read(5,input_HYM)
	read(5,input_numerics)

	close(5)

	! load coils (including CS) from separate files

	open(13,file='coil_input.dat')

	do i = 1, n_coils

		read(13,*) R_coil_temp(i), Z_coil_temp(i), coil_labels(i)

	enddo

	close(13)


	open(13,file='CS_input.dat')

	do i = 1, n_CS

		read(13,*) R_CS(i), Z_CS(i)

	enddo

	close(13)


	! sort coils
	! NOTE: coils are stored in the order they appear in the input, not by labels

	i_loc = 0
	n_repeated = 0

	do i = 1, n_coils

		label_loc = coil_labels(i)

		if(label_loc==0) then
			cycle
		endif

		i_loc = i_loc + 1

		R_coil(i_loc,1) = R_coil_temp(i)
		Z_coil(i_loc,1) = Z_coil_temp(i)

		j_loc = 1

		do j = i+1, n_coils

			if(coil_labels(j) == 0) then
				cycle
			endif

			if(coil_labels(j)==i_loc) then

				n_repeated = n_repeated + 1
				j_loc = j_loc + 1
				R_coil(i_loc,j_loc) = R_coil_temp(j)
				Z_coil(i_loc,j_loc) = Z_coil_temp(j)

				coil_labels(j) = 0	! this labels coils that have already been used

			endif

		enddo

		repeat_index(i_loc) = j_loc

	enddo

	n_currents = n_coils - n_repeated

	! central solenoid

	if(which_data==0) then
		call init_radius_HYM
	elseif(which_data==1) then
		call init_r0_IMSL
	endif
!	call array_allocation

!	if(which_data==1) then
!		call FLOW_equilibrium
!	endif

!	call set_grid

	continue

	return

end subroutine readinput


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine init_r0_IMSL
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! initialize plasma radius for FLOW input.

	use mod_parameters, only : theta_points_max_lim
	use mod_plasma, only : r_data, r_cscoef, r_ord, theta_points
	use pseudo_IMSL, only : DBSNAK, DBSINT

	real(kind=dkind), dimension(theta_points_max_lim,2) :: dummy
	integer :: i

	i = 0

	open(33,file='roftheta.dat', status='old', action='read')

	do 

		i = i+1
		read(33,*,end=99) dummy(i,1),dummy(i,2)

	enddo

99	continue

	close(33)

	theta_points = i-1

	allocate(r_data(theta_points+r_ord,3))
	allocate(r_cscoef(1,theta_points))

	do i=1,theta_points

		r_data(i,1) = dummy(i,1)
		r_data(i,2) = dummy(i,2)

	enddo


	call DBSNAK(theta_points, r_data(1:theta_points,1),  &
					r_ord,r_data(:,3))

	call DBSINT(theta_points, r_data(1:theta_points,1),  &
		 r_data(1:theta_points,2), r_ord,  &
		 r_data(:,3),  &
		 r_cscoef(1,1:theta_points))

	continue

	return

  end subroutine init_r0_IMSL



!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine init_radius_HYM
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! initializes minor radius for plasma edge

	use mod_parameters, only : dkind, theta_points_max_lim
	use pseudo_IMSL, only : dbsnak, dbsint
	use mod_plasma, only : theta_points

	implicit none

	integer :: i
	real(kind=dkind), dimension(theta_points_max_lim,2) :: dummy_edge

	i = 0

	open(33,file='edge_new.dat', status='old', action='read')

	do 

		i = i+1
		read(33,*,end=99) dummy_edge(i,1),dummy_edge(i,2)

	enddo

99	continue

	close(33)

	theta_points = i-1

	call plasma_shape_conversion(dummy_edge(1:theta_points,1),dummy_edge(1:theta_points,2))

	continue

	return

end subroutine init_radius_HYM

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine plasma_shape_conversion(R_b, Z_b)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! this subroutine converts an input of the kind (R,Z) to a (theta,r) form
! the output is the same as from init_r0_IMSL

	use mod_parameters, only : pi
	use pseudo_IMSL, only : DBSNAK, DBSINT, interp_check
	use mod_plasma, only :  r_data, r_cscoef, r_ord, theta_points, bigR_edge_data, Z_edge_data, bigR_edge_cscoef,  &
												Z_edge_cscoef
	use mod_output, only : indexx

	implicit none

	real(kind=dkind), dimension(theta_points) :: R_b, Z_b ! the boundary
	real(kind=dkind), dimension(theta_points) :: R_surf1, Z_surf1 ! the first "inside" magnetic surface
	real(kind=dkind) :: rloc, thetaloc
	integer, allocatable, dimension(:) :: sorter, i_repeated
	real(kind=dkind), allocatable, dimension(:,:) :: data_temp
	real(kind=dkind), allocatable, dimension(:) :: R_b_full, Z_b_full, R_surf1_full, Z_surf1_full
	real(kind=dkind), allocatable, dimension(:,:) :: bigR_surf1_temp, Z_surf1_temp
	real(kind=dkind) :: theta_min, theta_max
	real(kind=dkind) :: theta1, theta2, tol
	real(kind=dkind), dimension(3) :: th_search
	integer :: theta_points_temp

	integer :: i, j, k
	integer :: ind1, ind2, i0, iskip, ind_loc, ind_loc2, ind_min

	! since HYM works in cm, turn everything into meters
	do i = 1, theta_points
		R_b(i) = R_b(i)*1.d-2
		Z_b(i) = Z_b(i)*1.d-2
	enddo

	rmajor = (maxval(R_b)+minval(R_b))/2.d0

	theta_points_temp = theta_points

	! first create the outer radius

	allocate(data_temp(theta_points+1,2))

	do i=1,theta_points

		thetaloc = atan2(Z_b(i),R_b(i)-rmajor)

		if (thetaloc<0.d0) thetaloc = thetaloc + 2.d0*pi

		rloc = sqrt( (R_b(i)-rmajor)**2 + Z_b(i)**2 )

		data_temp(i,1) = thetaloc
		data_temp(i,2) = rloc

	enddo

	! first check if the data range covers a circle

	theta_min = minval(data_temp(1:theta_points,1))
	theta_max = maxval(data_temp(1:theta_points,1))

	if(theta_max-theta_min<2.d0*pi) then

		j = 1
		i = 0
		do while(j>0)
			i = i+1
			if(data_temp(i,1)==theta_min) then
				j = 0
			endif
		enddo

!		i = minloc(data_temp(1:theta_points,1))

		data_temp(theta_points+1,1) = data_temp(i,1) + 2.d0*pi
		data_temp(theta_points+1,2) = data_temp(i,2)

		allocate(R_b_full(theta_points+1))
		allocate(Z_b_full(theta_points+1))

		do i = 1, theta_points

			R_b_full(i) = R_b(i)
			Z_b_full(i) = Z_b(i)

		enddo

		i = 0

		do

			i = i+1

			if(theta_min == data_temp(i,1)) then

				data_temp(theta_points+1,2) = data_temp(i,2)
				ind_min = i
				R_b_full(theta_points+1) = R_b(i)
				Z_b_full(theta_points+1) = Z_b(i)
				exit

			endif

		enddo

		theta_max = theta_min + 2.d0*pi
		data_temp(theta_points+1,1) = theta_max

		theta_points = theta_points + 1

	else

		allocate(R_b_full(theta_points))
		allocate(Z_b_full(theta_points))

		do i = 1, theta_points

			R_b_full(i) = R_b(i)
			Z_b_full(i) = Z_b(i)

		enddo

	endif

	! reorder the data

	allocate(sorter(theta_points))

	call indexx(theta_points, data_temp(1:theta_points,1), sorter)

	! now check for repeated entries
	! this can happen if the input has a starting point different from theta=0

	iskip = 0
	allocate(i_repeated(theta_points))

	do i = 2,theta_points

		if(data_temp(sorter(i),1)==data_temp(sorter(i-1),1)) then

			iskip = iskip + 1
			i_repeated(iskip) = i

		endif

	enddo

	theta_points = theta_points - iskip

	allocate(r_data(theta_points+r_ord,3))
	allocate(r_cscoef(1,theta_points))
	allocate(bigR_edge_data(theta_points+r_ord,3))
	allocate(bigR_edge_cscoef(theta_points))
	allocate(Z_edge_data(theta_points+r_ord,3))
	allocate(Z_edge_cscoef(theta_points))

	j = 1

	do i = 1,theta_points+iskip

		if(i==i_repeated(j)) then
		! skip this point

			j = j+1

		else

			r_data(i-j+1,1) = data_temp(sorter(i),1)
			r_data(i-j+1,2) = data_temp(sorter(i),2)

			ind_loc = i-j+1

			bigR_edge_data(ind_loc,1) = data_temp(sorter(i),1)
			bigR_edge_data(ind_loc,2) = R_b_full(sorter(i))

			Z_edge_data(ind_loc,1) = data_temp(sorter(i),1)
			Z_edge_data(ind_loc,2) = Z_b_full(sorter(i))

		endif

	enddo

	deallocate(i_repeated)
	deallocate(sorter)
	deallocate(data_temp)

	call interp_setup(theta_points,r_ord, &
		r_data(1:theta_points,1),r_data(1:theta_points,2), &
		r_data(1:theta_points+r_ord,3),r_cscoef(1,1:theta_points))

	call interp_setup(theta_points,r_ord, &
		bigR_edge_data(1:theta_points,1),bigR_edge_data(1:theta_points,2), &
		bigR_edge_data(1:theta_points+r_ord,3),bigR_edge_cscoef(1:theta_points))

	call interp_setup(theta_points,r_ord, &
		Z_edge_data(1:theta_points,1),Z_edge_data(1:theta_points,2), &
		Z_edge_data(1:theta_points+r_ord,3),Z_edge_cscoef(1:theta_points))

	continue

!!$	deallocate(i_repeated)
!!$	deallocate(sorter)
!!$	deallocate(data_temp)
!!$	deallocate(bigR_surf1_temp)
!!$	deallocate(Z_surf1_temp)

!!$	call interp_setup(theta_points,r_ord, &
!!$		surf1_data(1:theta_points,1),surf1_data(1:theta_points,2), &
!!$		surf1_data(1:theta_points+r_ord,3),surf1_cscoef(1:theta_points))

!!$	call interp_setup(surf1_size,r_ord, &
!!$		bigR_surf1_data(1:surf1_size,1),bigR_surf1_data(1:surf1_size,2), &
!!$		bigR_surf1_data(1:surf1_size+r_ord,3),bigR_surf1_cscoef(1:surf1_size))

!!$	call interp_setup(surf1_size,r_ord, &
!!$		Z_surf1_data(1:surf1_size,1),Z_surf1_data(1:surf1_size,2), &
!!$		Z_surf1_data(1:surf1_size+r_ord,3),Z_surf1_cscoef(1:surf1_size))

	continue

end subroutine plasma_shape_conversion

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine interp_setup(npoints,int_ord,xdata,ydata,data3,cscoef)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! general routine to set up interpolations

	use pseudo_IMSL, only : DBSNAK, DBSINT

	implicit none

	integer :: int_ord	! order of the interpolation
	integer :: npoints	! dimension of the input data

	real(kind=dkind) :: xdata(npoints)	! the "x" data
	real(kind=dkind) :: ydata(npoints)	! the "y" data
	real(kind=dkind) :: data3(npoints+int_ord)	! the "x" points in the interpolation
	real(kind=dkind) :: cscoef(npoints)	! the interpolation coefficients

	call DBSNAK(npoints, xdata, int_ord,data3)

	call DBSINT(npoints, xdata, ydata, int_ord, data3, cscoef)

	continue

	return

end subroutine interp_setup





end module mod_input
