module mod_output

	use mod_parameters, only : dkind

	implicit none

	contains

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine writeoutput
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	use mod_parameters, only : pi
	use mod_coils, only : n_currents, n_CS, R_coil, Z_coil, I_coil, I_CS, repeat_index
	use mod_boundary, only : output_boundary_coord, n_boundary, psi_boundary

	real(kind=dkind), dimension(:,:), allocatable :: psi_data
	integer, allocatable, dimension(:) :: sorter, i_repeated

	integer :: iskip
	integer :: i, j

	! save the coil currents

	open(69, file='coils_out.dat')

		do i = 1, n_currents
		do j = 1, repeat_index(i)

			write(69,*) R_coil(i,j), Z_coil(i,j), I_coil(i)

		enddo
		enddo

		write(69,*) n_CS, I_CS

	close(69)

	open(69, file='RZ_out.dat')

	do i = 1, n_boundary

		write(69,*) output_boundary_coord(i,1), output_boundary_coord(i,2), psi_boundary(i)

	enddo

	close(69)

	! reorder the data

	allocate(sorter(n_boundary))

	call indexx(n_boundary, output_boundary_coord(1:n_boundary,0), sorter(1:n_boundary))

	! now check for repeated entries
	! this can happen if the input has a starting point different from theta=0

	iskip = 0
	allocate(i_repeated(n_boundary))

	do i = 2,n_boundary

		if(output_boundary_coord(sorter(i),0)==output_boundary_coord(sorter(i-1),0)) then

			iskip = iskip + 1
			i_repeated(iskip) = i

		endif

	enddo

	n_boundary = n_boundary - iskip

	allocate(psi_data(0:n_boundary+1,2))

	j = 1

	do i = 1,n_boundary+iskip

		if(i==i_repeated(j)) then
		! skip this point

			j = j+1

		else

			psi_data(i-j+1,1) = output_boundary_coord(sorter(i),0)
			psi_data(i-j+1,2) = psi_boundary(sorter(i))

		endif

	enddo

	deallocate(i_repeated)
	deallocate(sorter)

	psi_data(0,1) = psi_data(n_boundary,1) - 2.d0 * pi
	psi_data(0,2) = psi_data(n_boundary,2)

	psi_data(n_boundary+1,1) = psi_data(1,1) + 2.d0 * pi
	psi_data(n_boundary+1,2) = psi_data(1,2)

	open(69, file='psioftheta_out.dat')

	do i = 0, n_boundary+1

		write(69,*) psi_data(i,1), psi_data(i,2)

	enddo

	close(69)

	deallocate(psi_data)

	continue


end subroutine writeoutput



!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine indexx(n,arrin,indx)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! from numerical recipes

	implicit none

	integer :: n
	integer, dimension(1:n) :: indx
	real(kind=dkind), dimension(1:n) :: arrin

	integer :: i, j, l, ir, indxt
	real(kind=dkind) :: q

      do 11 j=1,n
        indx(j)=j
11    continue
      l=n/2+1
      ir=n
10    continue
        if(l.gt.1)then
          l=l-1
          indxt=indx(l)
          q=arrin(indxt)
        else
          indxt=indx(ir)
          q=arrin(indxt)
          indx(ir)=indx(1)
          ir=ir-1
          if(ir.eq.1)then
            indx(1)=indxt
            return
          endif
        endif
        i=l
        j=l+l
20      if(j.le.ir)then
          if(j.lt.ir)then
            if(arrin(indx(j)).lt.arrin(indx(j+1)))j=j+1
          endif
          if(q.lt.arrin(indx(j)))then
            indx(i)=indx(j)
            i=j
            j=j+j
          else
            j=ir+1
          endif
        go to 20
        endif
        indx(i)=indxt
      go to 10
 
end subroutine indexx



end module mod_output