module mod_plasma

	use mod_parameters, only : dkind

	implicit none

	real(kind=dkind) :: x_size, z_size, rcenter, zcenter, rmajor
	! the size of the equilibrium grid in the R and Z direction,
	! the geometric center of the grid and of the plasma

	integer :: nx, nz
	! points in the plasma grid

	real(kind=dkind) :: dx, dz

	real(kind=dkind), dimension(:), allocatable :: dx_a, dz_a
	! these arrays are set once for each grid

	integer, dimension(:,:,:), allocatable :: sort_grid
	!  real(kind=dkind), dimension(:,:,:), allocatable :: sort_grid
	!distinguishes between internal and external grid points;

	real(kind=dkind), dimension(:), allocatable :: x_coord, z_coord

	integer :: r_ord = 3
	real(kind=dkind), dimension(:,:), allocatable :: r_data
	real(kind=dkind), dimension(:,:), allocatable :: r_cscoef
	! the shape of the plasma boundary

	integer theta_points

	real(kind=dkind), dimension(:,:), allocatable :: psi, j_phi

	real(kind=dkind), dimension(:), allocatable ::  xknot_psi, zknot_psi
	real(kind=dkind), dimension(:,:), allocatable ::  psi_bscoef
	integer :: s_ord = 2
	! for psi interpolation, used for gradients at the edge

	real(kind=dkind), dimension(:,:,:), allocatable :: grad_psi_data
	real(kind=dkind), dimension(:,:), allocatable :: grad_psi_cscoef
	integer :: gpsi_ord = 3 ! interpolation order
	! grad psi on LCFS, LAST index (1-2) is coordinate

	real(kind=dkind), dimension(:), allocatable :: norm_grad
	real(kind=dkind), dimension(:,:), allocatable :: gpsi_norm_cscoef
	integer :: gpsi_norm_ord = 3 ! interpolation order
	! normal component of grad psi on LCFS

	real(kind=dkind), dimension(:,:), allocatable :: bigR_edge_data
	real(kind=dkind), dimension(:), allocatable :: bigR_edge_cscoef
	real(kind=dkind), dimension(:,:), allocatable :: Z_edge_data
	real(kind=dkind), dimension(:), allocatable :: Z_edge_cscoef
	! plasma edge R,Z coordinates

	real(kind=dkind), dimension(:,:), allocatable :: bigR_surf1_data
	real(kind=dkind), dimension(:), allocatable :: bigR_surf1_cscoef
	real(kind=dkind), dimension(:,:), allocatable :: Z_surf1_data
	real(kind=dkind), dimension(:), allocatable :: Z_surf1_cscoef
	! first inner surface R,Z coordinates

	integer :: surf1_size

	contains

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine read_equilibrium
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	use mod_parameters, only : which_data

	if(which_data==0) then
		call read_equilibrium_HYM
	elseif(which_data==1) then
		call read_equilibrium_FLOW
	endif

	continue

end subroutine read_equilibrium

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine read_equilibrium_HYM
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	use mod_parameters, only : HYM_current

	real(kind=dkind), dimension(:,:), allocatable :: dummy_array

	integer :: i, j

	open(unit=13,file='j_tot.dat')

	read(13,*) nz, nx

	call set_grid_HYM

	allocate(j_phi(1:nx,1:nz))
	allocate(dummy_array(1:nx,1:nz))

	read(13,*) ((dummy_array(j,i),i=1,nz),j=1,nx)
	read(13,*) ((dummy_array(j,i),i=1,nz),j=1,nx)
	read(13,*) ((j_phi(j,i),i=1,nz),j=1,nx)
	close(13)

!!$	read(13,*) ((dummy_array(i,j),i=1,nx),j=1,nz)
!!$	read(13,*) ((dummy_array(i,j),i=1,nx),j=1,nz)
!!$	read(13,*) ((j_phi(i,j),i=1,nx),j=1,nz)
!!$	close(13)

!!$	read(13,*) ((dummy_array(i,j),i=1,nz),j=1,nx)
!!$	read(13,*) ((dummy_array(i,j),i=1,nz),j=1,nx)
!!$	read(13,*) ((j_phi(i,j),i=1,nz),j=1,nx)
!!$	close(13)

    open (unit=69,file='J_HYM.plt')

    write(69,*)'TITLE="J_phi"'
    write(69,*)'Variables =" x ","y", "var"'		!, 	fname
    write(69,*)'ZONE I=',nx,',J=',nz,',F=Point'

	do j = 1, nz
	do i = 1, nx
		j_phi(i,j) = j_phi(i,j) * HYM_current
		write(69,*) x_coord(i), z_coord(j), j_phi(i,j)
	enddo
	enddo

	close(69)

	continue

!10  format(4(e10.4, 3x))
10   format(5es12.3)

end subroutine read_equilibrium_HYM

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine read_equilibrium_FLOW
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	allocate(psi(1:nx,1:nz))
	allocate(j_phi(1:nx,1:nz))

	call set_grid
	call read_FLOW_restart_data
	call get_J_phi

	continue

end subroutine read_equilibrium_FLOW

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine read_FLOW_restart_data
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! this straight from FLOW, reads the data saved for restart

	integer :: ierr
	integer i,j

    open (unit=17, file='psibin.out', form='unformatted', status='old', action='read')

	do j=1,nz
	do i=1,nx

		read(17) psi(i,j)
!		if(sort_grid(i,j,0)<1) psi(i,j) = 0.d0

	enddo
	enddo

	close(17)

	continue

	return

end subroutine read_FLOW_restart_data

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine get_J_phi
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! uses centered finite differences

	use mod_parameters, only : mu_mag

	real(kind=dkind) :: x
	real(kind=dkind) :: dx2, dz2
	integer :: i, j

    dx2 = dx*dx
    dz2 = dz*dz

	do j = 1, nz
	do i = 1, nx

		if(sort_grid(i,j,0)<2) then

			! external point
			j_phi(i,j) = 0

		else

			! internal point

			x = x_coord(i)

			j_phi(i,j) = ( -( (psi(i+1,j)-psi(i,j))/(x+dx/2.d0) - (psi(i,j)-psi(i-1,j))/(x-dx/2.d0) )/dx2  &
										- (psi(i,j+1)-2.d0*psi(i,j)+psi(i,j-1))/dz2/x ) / mu_mag

			continue

		endif

	enddo
	enddo

	continue

end subroutine get_J_phi


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine set_grid_HYM()
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! this defaults to uniform grid, different options may be implemented later

	use mod_parameters, only : HYM_length

	integer :: i, j, m, p, q

	real(kind=dkind) :: x_L, x_R
	real(kind=dkind) :: L1, L2, L3
	real(kind=dkind) :: ex,ez
	real(kind=dkind) :: temp, dummy, rminor

	if(allocated(x_coord)) deallocate(x_coord)
	if(allocated(z_coord)) deallocate(z_coord)

	if(allocated(dx_a)) deallocate(dx_a)
	if(allocated(dz_a)) deallocate(dz_a)

	allocate(x_coord(0:nx+1))
	allocate(z_coord(0:nz+1))

	x_coord=0.d0
	z_coord=0.d0

	allocate(dx_a(1:nx))
	allocate(dz_a(1:nz))

	read(13,*) (z_coord(i),i=1,nz),(x_coord(j),j=1,nx)

	do i = 1, nx
		x_coord(i) = x_coord(i) * HYM_length
	enddo

	do j = 1, nz
		z_coord(j) = z_coord(j) * HYM_length
	enddo

	do i=2,nx

		dx_a(i-1) = x_coord(i) - x_coord(i-1)

	enddo

	dx_a(nx) = dx_a(nx-1)

	do i=2,nz

		dz_a(i-1) = z_coord(i) -  z_coord(i-1)

	enddo

	dz_a(nz) = dz_a(nz-1)

	! assign grid point location
	! 1 = internal, -1 = external, 0 = on the edge (or very very close),
	! 2 = internal and not adjacent to any external point

	if(allocated(sort_grid)) deallocate(sort_grid)

	x_size = x_coord(nx) - x_coord(1)
	z_size = z_coord(nz) - z_coord(1)

	dx = x_size/(nx - 1.0d0)
	dz = z_size/(nz - 1.0d0)

	x_L = x_coord(1)
!	z_coord(1)

	x_coord(0) = x_coord(1) - dx_a(1)
	z_coord(0) = z_coord(1) - dz_a(1)

	allocate(sort_grid(0:nx+1,0:nz+1,0:0))

	call set_sort_grid(nx,nz)
	! this has been moved to a separate routine
	! to simplify things in the separatrix case

	continue

!10  format(4(e10.4, 3x))
10   format(5es12.3)

!	open(33,file='grid.plt')
!
!	write(33,*)'TITLE="grid"'
!	write(33,*)'Variables =" R [m] ","z [m]", "boh"'
!	write(33,*)'ZONE I=',nx,',J=',nz,',F=Point'
!
!	do j = 1,nz
!	do i = 1,nx
!
!		write(33,*) x_coord(i), z_coord(j),  sort_grid(i,j,0)
!
!	enddo
!	enddo
!
!	close(33)


	return

end subroutine set_grid_HYM


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine set_grid()
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! this defaults to uniform grid, different options may be implemented later

	integer :: i, j, m, p, q

	real(kind=dkind) :: x_L, x_R
	real(kind=dkind) :: L1, L2, L3
	real(kind=dkind) :: ex,ez
	real(kind=dkind) :: temp, dummy, rminor

	if(allocated(x_coord)) deallocate(x_coord)
	if(allocated(z_coord)) deallocate(z_coord)

	if(allocated(dx_a)) deallocate(dx_a)
	if(allocated(dz_a)) deallocate(dz_a)

	allocate(x_coord(0:nx+1))
	allocate(z_coord(0:nz+1))

	allocate(dx_a(0:nx+1))
	allocate(dz_a(0:nz+1))

	dx = x_size/(nx - 1.0d0)
	dz = z_size/(nz - 1.0d0)

	x_L = rcenter - 0.5d0*x_size
	x_R = rcenter + 0.5d0*x_size

	dx_a = dx
	dz_a = dz

	x_coord(1) = x_L
	z_coord(1) = zcenter - 0.5d0*z_size 

	x_coord(0) = x_coord(1) - dx_a(1)
	z_coord(0) = z_coord(1) - dz_a(1)

	do i=2,nx+1

		x_coord(i) = x_coord(i-1) + dx_a(i-1)

	enddo

	do i=2,nz+1

		z_coord(i) = z_coord(i-1) + dz_a(i-1)

	enddo

	!check grid

	if ( (dabs(x_coord(nx)-x_R)/x_R>1.d-12).or.  &
		(dabs(z_coord(nz)-z_coord(1)-z_size)/z_size>1.d-12) ) then

		print*, 'problem in set_grid:'
		print*, 'x_R=', x_R
		print*, 'x_coord(nx)=', x_coord(nx)
		print*, 'z_size=', z_size
		print*, 'z_coord(nz)=', z_coord(nz)
		pause
		stop

	endif

	! assign grid point location
	! 1 = internal, -1 = external, 0 = on the edge (or very very close),
	! 2 = internal and not adjacent to any external point

	if(allocated(sort_grid)) deallocate(sort_grid)

	allocate(sort_grid(0:nx+1,0:nz+1,0:0))

	call set_sort_grid(nx,nz)
	! this has been moved to a separate routine
	! to simplify things in the separatrix case

	continue

	open(33,file='grid.plt')

	write(33,*)'TITLE="grid"'
	write(33,*)'Variables =" R [m] ","z [m]", "boh"'
	write(33,*)'ZONE I=',nx,',J=',nz,',F=Point'

!	do j = 1,nz
!	do i = 1,nx
!
!		write(33,*) x_coord(i), z_coord(j),  sort_grid(i,j,0)
!
!	enddo
!	enddo
!
!	close(33)


	return

end subroutine set_grid

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine set_sort_grid(nx,nz)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! sorts out which grid points are inside the plasma edge.

	real(kind=dkind), parameter :: small = 1.d-2
	logical :: inner

	integer :: nx,nz
	integer :: i,j,k
	real(kind=dkind) :: ex, ez, rminor, dx, dz, dummy

	sort_grid = -1

	do j=1,nz
	do i=1,nx

		call radius(i,j,nx,nz,ex,ez,rminor,dx,dz)

		if((ex*ex + ez*ez) > rminor**2) then

			sort_grid(i,j,0) = -1

		elseif( (ex*ex + ez*ez) >= (rminor - small*sqrt(dx_a(i)**2+dz_a(j)**2) )**2 ) then

			sort_grid(i,j,0) = 0

		else

			sort_grid(i,j,0) = 1

		end if

	enddo
	enddo

	! mark the really inner points

	do j = 1, nz
	do i = 1, nx

		if (sort_grid(i,j,0)==1) then

			inner = .true.

			do k = -1, 1, 2

				if (sort_grid(i+k,j,0)<1) inner = .false.
				if (sort_grid(i,j+k,0)<1) inner = .false.

			enddo

			if(inner) then
				sort_grid(i,j,0)=2
			endif

		endif

	enddo
	enddo

	continue

end subroutine set_sort_grid

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine radius(i,j,nx,nz,ex,ez,r,dxx,dzz)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! this defaults to FLOW tri_type==8. Other options may be implemented later.
! see FLOW documentation for details.

	! this subroutine computes the minor radius r of the plasma
	! as a function of the angle AND of the triangularity option

	use mod_parameters, only : pi
	use pseudo_IMSL, only : dbsval

	implicit none

	integer, intent(in) :: i,j,nx,nz
	real (kind=dkind) :: dxx,dzz,bigR,Z, diff
	real (kind=dkind), intent(out) :: ex,ez,r
	integer k
	real (kind=dkind) :: r_in,r_out,angle


	if( (i==0).or.(j==0).or.(i>nx).or.(j>nz) ) then
	! somehow the code got here with an index exceeding the grid dimension
	! assign this to be an external point

		ex = 1.d0
		ez = 1.d0
		r = 1.d-2
		return

	endif

	ex = x_coord(i)-rmajor
	ez = z_coord(j)

	if (ex==0.d0) then
		angle = pi/2.d0 * dsign(1.d0,ez)
	else
		angle = datan2(ez,ex)
	endif

	r = 0.d0
	bigR = 0.d0
	Z = 0.d0

	if(angle>=r_data(theta_points,1)) then

		angle = angle - 2.d0*pi

	elseif(angle<r_data(1,1)) then

		angle = angle + 2.d0*pi

	endif

	r = dbsval(angle, r_ord, r_data(:,3),  &
				theta_points, r_cscoef(1,1:theta_points) )


	continue

end subroutine radius


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine radius_theta(angle,r,x,z)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! straight from FLOW

! this subroutine computes the minor radius r of the plasma
! as a function of the angle AND of the triangularity option

	use mod_parameters, only : pi
	use pseudo_IMSL, only : dbsval

	real (kind=dkind), intent(out) :: r,x,z
	real(kind=dkind) :: angle
	real(kind=dkind) :: theta

	r = 0.d0
	theta = angle

	if(theta>=r_data(theta_points,1)) then

		theta = theta - 2.d0*pi

	elseif(theta<r_data(1,1)) then

		theta = theta + 2.d0*pi

	endif

	r = dbsval(theta, r_ord, r_data(:,3),  &
		theta_points, r_cscoef(1,1:theta_points) )

	x = rmajor + r*cos(angle)
	z = r*sin(angle)

	continue

end subroutine radius_theta


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine radius_theta_with_ders(angle,r,x,z,rprim,bigRprim,Zprim)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! straight from FLOW

! this subroutine computes the minor radius r of the plasma
! as a function of the angle AND of the triangularity option

	use mod_parameters, only : pi
	use pseudo_IMSL, only : dbsval, dbsder

	real (kind=dkind), intent(out) :: r,x,z,rprim,bigRprim,Zprim
	real(kind=dkind) :: angle
	real(kind=dkind) :: theta, cloc, sloc

	r = 0.d0
	theta = angle

	if(theta>=r_data(theta_points,1)) then

		theta = theta - 2.d0*pi

	elseif(theta<r_data(1,1)) then

		theta = theta + 2.d0*pi

	endif

	r = dbsval(theta, r_ord, r_data(:,3),  &
		theta_points, r_cscoef(1,1:theta_points) )

	cloc = cos(angle)
	sloc = sin(angle)

	x = rmajor + r*cloc
	z = r*sloc

	rprim = dbsder(1,theta, r_ord, r_data(:,3),  &
			theta_points, r_cscoef(1,1:theta_points) )

	bigRprim = rprim*cloc - r*sloc

	Zprim = rprim*sloc + r*cloc

	continue

end subroutine radius_theta_with_ders

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine theta_of_RZ(angle,R,Z)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	use mod_parameters, only : pi

	real(kind=dkind) :: angle,R,Z
	real(kind=dkind) :: ex, ez

	ex = R-rmajor
	ez = Z

	if (ex==0.d0) then
		angle = pi/2.d0 * dsign(1.d0,ez)
	else
		angle = datan2(ez,ex)
	endif

	if(angle<0.d0) then
		angle = angle + 2.d0*pi
	endif

	continue

end subroutine theta_of_RZ



end module mod_plasma