module mod_transversal_commons

	use mod_parameters, only: dkind, r_of_theta_points
	use mod_arrays, only : theta_edge, fixed_coords, shape_grad
	use mod_plasma, only:  grad_psi_data, grad_psi_cscoef, gpsi_ord, norm_grad,  &
											gpsi_norm_cscoef, gpsi_norm_ord

	implicit none

	real(kind=dkind) :: bigR_loc, Z_loc
	real(kind=dkind) :: bigR_surf1, bigZ_surf1 !points on surf1, used when global variables are needed
	real(kind=dkind) :: c0_eq, c1_eq
	! coefficients for the normal line Z = c0 + c1 R

	integer :: normal_case
	! -1 -> vertical normal
	! 0 -> standard case
	! 1 -> horizontal normal


	real(kind=dkind) :: small = 1.d-6

    integer :: itemax = 200
    real(kind=dkind) :: xtoll = 1.d-6
    real(kind=dkind) :: ftoll = 1.d-6
    integer :: error


	contains

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine get_psi_gradient
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	use mod_parameters, only : which_data

	if(which_data==0) then
		call get_psi_gradient_HYM
	elseif(which_data==1) then
		call get_psi_gradient_FLOW
	endif

	return

end subroutine get_psi_gradient

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine get_psi_gradient_HYM
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! uses last two surfaces to compute gradients

	use mod_parameters, only : pi, r_of_theta_points
	use mod_plasma, only: r_ord, bigR_edge_data, bigR_edge_cscoef, Z_edge_data, Z_edge_cscoef,  &
											theta_points, norm_grad
	use pseudo_IMSL, only : dbsval

!	real(kind=dkind) :: thguessr, thguessl, thsol
	real(kind=dkind) :: dist1
	real(kind=dkind) :: psi1
	real(kind=dkind) :: thetaloc
	real(kind=dkind), dimension(:,:), allocatable :: grad_norm_debug
	integer :: i

	psi1 = 1.d0/20.d0 ! psi value on first surface

!          call secroot(findsurf,rguessr,rguessl,rloc,itemax,xtoll,ftoll,error)

	call gradient_mod_setup
	! sets up initial crap

	call get_edge_gradients
	! gets dR/dtheta, dZ/dtheta

	allocate(grad_norm_debug(0:r_of_theta_points,1:3))

	do i = 0, r_of_theta_points

		call get_normal(i)
		! gets normal, used next to calculate bigR_surf1, bigZ_surf1

		call get_surf_points(i)
		! gets bigR_surf1, bigZ_surf1

		thetaloc = theta_edge(i)

		if(thetaloc<bigR_edge_data(1,1)) then
			thetaloc = thetaloc + 2.d0*pi
		elseif(thetaloc>bigR_edge_data(theta_points,1)) then
			thetaloc = thetaloc - 2.d0*pi
		endif

		bigR_loc = dbsval(thetaloc, r_ord, bigR_edge_data(1:theta_points+r_ord,3),  &
									theta_points, bigR_edge_cscoef(:))

		Z_loc = dbsval(thetaloc, r_ord, Z_edge_data(1:theta_points+r_ord,3),  &
									theta_points, Z_edge_cscoef(:))

		dist1 = ((bigR_loc-bigR_surf1)**2+(Z_loc-bigZ_surf1)**2)**0.5d0

		norm_grad(i) = psi1/dist1

		grad_norm_debug(i,1) = bigR_loc
		grad_norm_debug(i,2) = Z_loc
		grad_norm_debug(i,3) = norm_grad(i)

		continue

	enddo

	return

end subroutine get_psi_gradient_HYM

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine get_psi_gradient_FLOW
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! uses interpolations to compute gradients

	use mod_parameters, only : pi, r_of_theta_points
	use mod_plasma, only: r_ord, bigR_edge_data, bigR_edge_cscoef, Z_edge_data, Z_edge_cscoef,  &
											theta_points, s_ord, xknot_psi, zknot_psi, psi_bscoef, nx, nz, norm_grad
!	use mod_input, only : psi_interp_setup
!	use mod_equation_system, only : fixed_coords
	use pseudo_IMSL, only : dbs2dr

!	real(kind=dkind) :: thguessr, thguessl, thsol
	real(kind=dkind) :: gx, gz
	real(kind=dkind) :: thetaloc
	real(kind=dkind), dimension(:,:), allocatable :: grad_norm_debug
	integer :: i

	call psi_interp_setup

	call gradient_mod_setup
	! sets up initial crap

!	call get_edge_gradients
	! gets dR/dtheta, dZ/dtheta

	allocate(grad_norm_debug(0:r_of_theta_points,1:3))

	do i = 1, r_of_theta_points

		thetaloc = theta_edge(i)

!!$		if(thetaloc<bigR_edge_data(1,1)) then
!!$			thetaloc = thetaloc + 2.d0*pi
!!$		elseif(thetaloc>bigR_edge_data(theta_points,1)) then
!!$			thetaloc = thetaloc - 2.d0*pi
!!$		endif
!!$
!!$		bigR_loc = dbsval(thetaloc, r_ord, bigR_edge_data(1:theta_points+r_ord,3),  &
!!$									theta_points, bigR_edge_cscoef(:))
!!$
!!$		Z_loc = dbsval(thetaloc, r_ord, Z_edge_data(1:theta_points+r_ord,3),  &
!!$									theta_points, Z_edge_cscoef(:))

		bigR_loc = fixed_coords(i,1)
		Z_loc = fixed_coords(i,2)

		gx = dbs2dr(1,0,bigR_loc,Z_loc,s_ord,s_ord,xknot_psi, zknot_psi,nx,nz,psi_bscoef)
		gz = dbs2dr(0,1,bigR_loc,Z_loc,s_ord,s_ord,xknot_psi, zknot_psi,nx,nz,psi_bscoef)

		norm_grad(i) = sqrt(gx**2+gz**2)

		grad_norm_debug(i,1) = bigR_loc
		grad_norm_debug(i,2) = Z_loc
		grad_norm_debug(i,3) = norm_grad(i)

		continue

	enddo

	return

end subroutine get_psi_gradient_FLOW

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine psi_interp_setup
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! sets up the interpolation of psi in the plasma, to be used for gradients etc.

	use mod_plasma, only : x_coord, z_coord, xknot_psi, zknot_psi, psi_bscoef, nx, nz, psi, s_ord
	use pseudo_IMSL, only : DBS2IN, DBS2GD, DBSNAK

	integer :: alloc_stat
	integer :: i, j

!-----------------------------------------
! interpolation set up

	allocate (psi_bscoef(1:nx,1:nz),stat = alloc_stat)

	allocate (xknot_psi(1:nx+s_ord),stat = alloc_stat)
	allocate (zknot_psi(1:nz+s_ord),stat = alloc_stat)

	call DBSNAK(nx,x_coord,s_ord,xknot_psi)
	call DBSNAK(nz,z_coord,s_ord,zknot_psi)

	! (these 2 define the nodes)

! end of interpolation set up
!-----------------------------------------
!-----------------------------------------
! set psi

	call DBS2IN(nx,x_coord,nz,z_coord,psi,nx,  &
				s_ord,s_ord,xknot_psi,zknot_psi,psi_bscoef(:,:) )

!-----------------------------------------

	continue

	return

end subroutine psi_interp_setup




!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine gradient_mod_setup
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! just sets up initial arrays to keep things clean

	use mod_parameters, only:  pi, solution_method, n_fit
	use mod_arrays, only : fixed_coords

	integer :: n_size
	integer :: i, istep, j

	r_of_theta_points = n_fit

	allocate(theta_edge(0:r_of_theta_points))
	allocate(grad_psi_data(0:r_of_theta_points+gpsi_ord,1:2,1:2))
	allocate(grad_psi_cscoef(0:r_of_theta_points,1:2))
	allocate(shape_grad(0:r_of_theta_points,1:2))
	allocate(norm_grad(0:r_of_theta_points+gpsi_norm_ord))
	allocate(gpsi_norm_cscoef(0:r_of_theta_points,1:2))


	if(solution_method==11) then
		n_size = n_fit
		istep = 1
	elseif(solution_method==12) then
		istep = 2
		n_size = n_fit*2
	endif

	do i = 1, n_size, istep

		j = i/istep

		theta_edge(j) = fixed_coords(i,0)

	enddo

	theta_edge(0) = theta_edge(j) - 2.d0*pi
!	theta_edge(0) = theta_edge(r_of_theta_points) - 2.d0*pi

	continue

end subroutine gradient_mod_setup

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine get_edge_gradients
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! calculates derivatives of R and Z with respect to theta

	use mod_parameters, only : pi
	use pseudo_IMSL, only: dbsder
	use mod_plasma, only: r_ord, bigR_edge_data, bigR_edge_cscoef, Z_edge_data, Z_edge_cscoef, theta_points

	real(kind=dkind) :: gx_loc, gz_loc, thetaloc
	integer :: i

	do i = 0, r_of_theta_points-1

		thetaloc = theta_edge(i)

		if(thetaloc<bigR_edge_data(1,1)) then
			thetaloc = thetaloc + 2.d0*pi
		elseif(thetaloc>bigR_edge_data(theta_points,1)) then
			thetaloc = thetaloc - 2.d0*pi
		endif

		gx_loc = dbsder(1,thetaloc, r_ord, bigR_edge_data(1:theta_points+r_ord,3),  &
								theta_points, bigR_edge_cscoef(:))

		shape_grad(i,1) = gx_loc

		gz_loc = dbsder(1,thetaloc, r_ord, Z_edge_data(1:theta_points+r_ord,3),  &
								theta_points, Z_edge_cscoef(:))

		shape_grad(i,2) = gz_loc

	enddo

	shape_grad(i,1) = shape_grad(0,1)
	shape_grad(i,2) = shape_grad(0,2)

	continue

end subroutine get_edge_gradients


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine get_surf_points(ind)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! gets bigR_surf1, bigZ_surf1 etc. (can use the same routine for all surface points)
! use secroot for theta

	use mod_parameters, only : pi
	use mod_plasma, only: r_ord, bigR_surf1_data, bigR_surf1_cscoef, Z_surf1_data, Z_surf1_cscoef,  &
							bigR_edge_data, bigR_edge_cscoef, Z_edge_data, Z_edge_cscoef, surf1_size, theta_points	
	use pseudo_IMSL, only: dbsval

	integer, intent(in) :: ind
	real(kind=dkind) :: theta_R, theta_L, theta_L1, theta_L2, thetaloc
	real(kind=dkind) :: fR, fL, fL1, fL2
	real(kind=dkind) :: dth

	thetaloc = theta_edge(ind)
	dth = pi/500.d0

	! fist set values on external surface, if needed
	if(normal_case == -1) then
	! vertical normal case

		if(thetaloc<bigR_edge_data(1,1)) then
			thetaloc = thetaloc + 2.d0*pi
		elseif(thetaloc>bigR_edge_data(theta_points,1)) then
			thetaloc = thetaloc - 2.d0*pi
		endif

		bigR_loc = dbsval(thetaloc, r_ord, bigR_edge_data(1:theta_points+r_ord,3),  &
									theta_points, bigR_edge_cscoef(:))

	elseif(normal_case == 1) then
	! horizontal normal case

		if(thetaloc<bigR_edge_data(1,1)) then
			thetaloc = thetaloc + 2.d0*pi
		elseif(thetaloc>bigR_edge_data(theta_points,1)) then
			thetaloc = thetaloc - 2.d0*pi
		endif

		Z_loc = dbsval(thetaloc, r_ord, Z_edge_data(1:theta_points+r_ord,3),  &
									theta_points, Z_edge_cscoef(:))

	endif

	theta_R = thetaloc
	fR = findtheta_wrapper(theta_R)

	theta_L1 = theta_R
	theta_L2 = theta_R
	fL = fR

	do while(fR*fL>0.d0)

		theta_L1 = theta_L1 + dth
		theta_L2 = theta_L2 - dth

!!$		if(theta_L1>bigR_edge_data(theta_points,1)) then
!!$			theta_L1 = theta_L1 - 2.d0*pi
!!$		endif

!!$		if(theta_L2<bigR_edge_data(1,1)) then
!!$			theta_L2 = theta_L2 + 2.d0*pi
!!$		endif

		fL1 = findtheta_wrapper(theta_L1)
		fL2 = findtheta_wrapper(theta_L2)

		if(fL2*fR<0.d0) then
			fL = fL2; theta_L = theta_L2
		else
			fL = fL1; theta_L = theta_L1
		endif

		continue

	enddo

	call secroot(findtheta_wrapper,theta_R,theta_L,thetaloc,itemax,xtoll,ftoll,error)



	if(error>0) then
		print*, 'error in secroot, get_surf_points'
	endif

	! note that this needs to be repeated because in principle the last point is not
	! necessarily the point returned as answer (in practice it should not happen)
	bigR_surf1 = dbsval(thetaloc, r_ord, bigR_surf1_data(1:surf1_size+r_ord,3),  &
								surf1_size, bigR_surf1_cscoef(:))

	bigZ_surf1 = dbsval(thetaloc, r_ord, Z_surf1_data(1:surf1_size+r_ord,3),  &
								surf1_size, Z_surf1_cscoef(:))

	continue

end subroutine get_surf_points


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine get_normal(ii)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! this returns the equation of the normal to the selected point on the edge
! since they are useful later, (r,Z) are also returned
! since we are using the same points as the points the gradients are defined on,
! it is sufficient to work with the index

	use mod_parameters, only : pi
	use mod_plasma, only: r_ord, bigR_edge_data, bigR_edge_cscoef, Z_edge_data, Z_edge_cscoef,  theta_points
	use pseudo_IMSL, only: dbsval

	integer :: ii
	real(kind=dkind) :: theta
	real(kind=dkind) :: gx_loc, gz_loc

	theta = theta_edge(ii)

	if(theta<bigR_edge_data(1,1)) then
		theta = theta + 2.d0*pi
	elseif(theta>bigR_edge_data(theta_points,1)) then
		theta = theta - 2.d0*pi
	endif

	gx_loc = shape_grad(ii,1)
	gz_loc = shape_grad(ii,2)

	bigR_loc = dbsval(theta, r_ord, bigR_edge_data(1:theta_points+r_ord,3),  &
								theta_points, bigR_edge_cscoef(:))

	Z_loc = dbsval(theta, r_ord, Z_edge_data(1:theta_points+r_ord,3),  &
								theta_points, Z_edge_cscoef(:))

	normal_case = 0

	if(abs(gx_loc*gz_loc) < small * (abs(gx_loc)+abs(gz_loc))) then
	! tangent is (very close to) either horizontal or vertical

		if(abs(gx_loc)>abs(gz_loc)) then
		! vertical normal (horizontal tangent)

			normal_case = -1

		else
		! horizontal normal (vertical tangent)

			normal_case = 1

		endif

	else
	! normal case

		normal_case = 0
		c1_eq = -gx_loc/gz_loc
		! tangent would be gz/gx, normal is given by = -1/[tangent]
		c0_eq = Z_loc - c1_eq*bigR_loc

	endif

end subroutine get_normal

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
function findtheta_wrapper(theta) result(answer)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! Finds theta of point on inner surface given the direction of the normal.
! Uses dedicated algorithms if the normal is parallel (or almost so) to either of the axes.

	use mod_parameters, only : pi
	use mod_plasma, only: r_ord, bigR_surf1_data, bigR_surf1_cscoef, Z_surf1_data, Z_surf1_cscoef, surf1_size,  &
										bigR_edge_data, bigR_edge_cscoef, Z_edge_data, Z_edge_cscoef, theta_points
	use pseudo_IMSL, only: dbsval

	real(kind=dkind) :: theta
	real(kind=dkind) :: answer
	real(kind=dkind) :: thetaloc

	thetaloc = theta

	if(normal_case == 0) then
	! normal normal case

		answer = findtheta(thetaloc)

	elseif(normal_case == -1) then
	! vertical normal case

!!$		if(thetaloc<bigR_edge_data(1,1)) then
!!$			thetaloc = thetaloc + 2.d0*pi
!!$		elseif(thetaloc>bigR_edge_data(theta_points,1)) then
!!$			thetaloc = thetaloc - 2.d0*pi
!!$		endif

!!$		bigR_loc = dbsval(thetaloc, r_ord, bigR_edge_data(1:theta_points+r_ord,3),  &
!!$									theta_points, bigR_edge_cscoef(:))

		answer = findtheta_vert(thetaloc)

	elseif(normal_case == 1) then
	! horizontal normal case

!!$		if(thetaloc<bigR_edge_data(1,1)) then
!!$			thetaloc = thetaloc + 2.d0*pi
!!$		elseif(thetaloc>bigR_edge_data(theta_points,1)) then
!!$			thetaloc = thetaloc - 2.d0*pi
!!$		endif

!!$		Z_loc = dbsval(thetaloc, r_ord, Z_edge_data(1:theta_points+r_ord,3),  &
!!$									theta_points, Z_edge_cscoef(:))

		answer = findtheta_hor(thetaloc)

	endif

	return

end function findtheta_wrapper

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
function findtheta(theta) result(answer)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! NEEDS EXPLANATION

	use mod_plasma, only: r_ord, bigR_surf1_data, bigR_surf1_cscoef, Z_surf1_data, Z_surf1_cscoef, surf1_size,  &
											theta_points
	use pseudo_IMSL, only: dbsval

	real(kind=dkind) :: theta
	real(kind=dkind) :: answer
	real(kind=dkind) :: thetaloc

	thetaloc = theta

	bigR_surf1 = dbsval(theta, r_ord, bigR_surf1_data(1:surf1_size+r_ord,3),  &
								surf1_size, bigR_surf1_cscoef(:))

	bigZ_surf1 = dbsval(theta, r_ord, Z_surf1_data(1:surf1_size+r_ord,3),  &
								surf1_size, Z_surf1_cscoef(:))

	answer = c0_eq + c1_eq * bigR_surf1 - bigZ_surf1

	return

end function findtheta

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
function findtheta_vert(theta) result(answer)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	use mod_plasma, only: r_ord, bigR_surf1_data, bigR_surf1_cscoef, Z_surf1_data, Z_surf1_cscoef, surf1_size,  &
											theta_points
	use pseudo_IMSL, only: dbsval

	real(kind=dkind) :: theta
	real(kind=dkind) :: answer
	real(kind=dkind) :: thetaloc

	thetaloc = theta

	bigR_surf1 = dbsval(thetaloc, r_ord, bigR_surf1_data(1:surf1_size+r_ord,3),  &
								surf1_size, bigR_surf1_cscoef(:))

	answer = bigR_loc - bigR_surf1

	return

end function findtheta_vert

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
function findtheta_hor(theta) result(answer)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

	use mod_plasma, only: r_ord, Z_surf1_data, Z_surf1_cscoef, surf1_size, theta_points
	use pseudo_IMSL, only: dbsval

	real(kind=dkind) :: theta
	real(kind=dkind) :: answer
	real(kind=dkind) :: thetaloc

	thetaloc = theta

	bigZ_surf1 = dbsval(thetaloc, r_ord, Z_surf1_data(1:surf1_size+r_ord,3),  &
								surf1_size, Z_surf1_cscoef(:))

	answer = Z_loc - bigZ_surf1

	return

end function findtheta_hor

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
subroutine secroot(fx,x1,x2,x,itemax,xtoll,ftoll,error)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! Uses secant method to find zero of given function fx(x); x is a scalar.

	implicit none

	real(kind=dkind) :: x1,x2,x
	integer :: i
	integer :: itemax
	real(kind=dkind) :: xtoll, ftoll
	real(kind=dkind) :: f1,f2,fx
	real(kind=dkind) :: m, x3, f3
	integer :: error

	external fx

	error = 0

	f1 = fx(x1)
	f2 = fx(x2)

	if(f1*f2>0.d0) then

!		pause 'error in secroot, root is not bracketed'
		error = 1
		return

	endif

	if(f2<f1) then
	! (f2>0 by hypothesis)

		x3 = x2
		x2 = x1
		x1 = x3

		f3 = f2
		f2 = f1
		f1 = f3

	endif




	sloop : do i=1,itemax

		m = (f2-f1)/(x2-x1)
		x3 = x1 - f1/m
		f3 = fx(x3)

		if(f3>0.d0) then

			x2 = x3
			f2 = f3

		else

			x1 = x3
			f1 = f3

		endif

		if( (abs(f2-f1)<ftoll).or.(abs(x2-x1)<xtoll).or.  &
			 (abs(f2)<ftoll).or.(abs(f1)<ftoll) ) then
		! converged!

			exit sloop

		endif

	enddo sloop

	if(abs(f2)<abs(f1)) then

		x = x2

	else

		x = x1

	endif

end subroutine secroot



end module mod_transversal_commons
